@@ -3,7 +3,7 @@
  *  Here we examine manipulations on GiNaC's symbolic matrices. */
 
 /*
- *  GiNaC Copyright (C) 1999-2015 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2016 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@ static unsigned matrix_determinants()
 	symbol g("g"), h("h"), i("i");
 	
 	// check symbolic trivial matrix determinant
-	m1.set(0,0,a);
+	m1 = matrix{{a}};
 	det = m1.determinant();
 	if (det != a) {
 		clog << "determinant of 1x1 matrix " << m1
@@ -46,8 +46,8 @@ static unsigned matrix_determinants()
 	}
 	
 	// check generic dense symbolic 2x2 matrix determinant
-	m2.set(0,0,a).set(0,1,b);
-	m2.set(1,0,c).set(1,1,d);
+	m2 = matrix{{a, b},
+	            {c, d}};
 	det = m2.determinant();
 	if (det != (a*d-b*c)) {
 		clog << "determinant of 2x2 matrix " << m2
@@ -56,9 +56,9 @@ static unsigned matrix_determinants()
 	}
 	
 	// check generic dense symbolic 3x3 matrix determinant
-	m3.set(0,0,a).set(0,1,b).set(0,2,c);
-	m3.set(1,0,d).set(1,1,e).set(1,2,f);
-	m3.set(2,0,g).set(2,1,h).set(2,2,i);
+	m3 = matrix{{a, b, c},
+	            {d, e, f},
+	            {g, h, i}};
 	det = m3.determinant();
 	if (det != (a*e*i - a*f*h - d*b*i + d*c*h + g*b*f - g*c*e)) {
 		clog << "determinant of 3x3 matrix " << m3
@@ -67,9 +67,9 @@ static unsigned matrix_determinants()
 	}
 	
 	// check dense numeric 3x3 matrix determinant
-	m3.set(0,0,numeric(0)).set(0,1,numeric(-1)).set(0,2,numeric(3));
-	m3.set(1,0,numeric(3)).set(1,1,numeric(-2)).set(1,2,numeric(2));
-	m3.set(2,0,numeric(3)).set(2,1,numeric(4)).set(2,2,numeric(-2));
+	m3 = matrix{{0, -1,  3},
+	            {3, -2,  2},
+	            {3,  4, -2}};
 	det = m3.determinant();
 	if (det != 42) {
 		clog << "determinant of 3x3 matrix " << m3
@@ -78,8 +78,8 @@ static unsigned matrix_determinants()
 	}
 	
 	// check dense symbolic 2x2 matrix determinant
-	m2.set(0,0,a/(a-b)).set(0,1,1);
-	m2.set(1,0,b/(a-b)).set(1,1,1);
+	m2 = matrix{{a/(a-b), 1},
+	            {b/(a-b), 1}};
 	det = m2.determinant();
 	if (det != 1) {
 		if (det.normal() == 1)  // only half wrong
@@ -101,9 +101,9 @@ static unsigned matrix_determinants()
 	}
 	
 	// check characteristic polynomial
-	m3.set(0,0,a).set(0,1,-2).set(0,2,2);
-	m3.set(1,0,3).set(1,1,a-1).set(1,2,2);
-	m3.set(2,0,3).set(2,1,4).set(2,2,a-3);
+	m3 = matrix{{a, -2,   2},
+		    {3, a-1,  2},
+		    {3,  4,  a-3}};
 	ex p = m3.charpoly(a);
 	if (p != 0) {
 		clog << "charpoly of 3x3 matrix " << m3
@@ -135,10 +135,9 @@ static unsigned matrix_invert1()
 static unsigned matrix_invert2()
 {
 	unsigned result = 0;
-	matrix m(2,2);
 	symbol a("a"), b("b"), c("c"), d("d");
-	m.set(0,0,a).set(0,1,b);
-	m.set(1,0,c).set(1,1,d);
+	matrix m = {{a, b},
+	            {c, d}};
 	matrix m_i = m.inverse();
 	ex det = m.determinant();
 	
@@ -157,13 +156,12 @@ static unsigned matrix_invert2()
 static unsigned matrix_invert3()
 {
 	unsigned result = 0;
-	matrix m(3,3);
 	symbol a("a"), b("b"), c("c");
 	symbol d("d"), e("e"), f("f");
 	symbol g("g"), h("h"), i("i");
-	m.set(0,0,a).set(0,1,b).set(0,2,c);
-	m.set(1,0,d).set(1,1,e).set(1,2,f);
-	m.set(2,0,g).set(2,1,h).set(2,2,i);
+	matrix m = {{a, b, c},
+	            {d, e, f},
+	            {g, h, i}};
 	matrix m_i = m.inverse();
 	ex det = m.determinant();
 	
@@ -187,36 +185,32 @@ static unsigned matrix_invert3()
 static unsigned matrix_solve2()
 {
 	// check the solution of the multiple system A*X = B:
-	//	 [ 1  2 -1 ] [ x0 y0 ]   [ 4 0 ]
-	//	 [ 1  4 -2 ]*[ x1 y1 ] = [ 7 0 ]
-	//	 [ a -2  2 ] [ x2 y2 ]   [ a 4 ]
+	//       [ 1  2 -1 ] [ x0 y0 ]   [ 4 0 ]
+	//       [ 1  4 -2 ]*[ x1 y1 ] = [ 7 0 ]
+	//       [ a -2  2 ] [ x2 y2 ]   [ a 4 ]
 	unsigned result = 0;
 	symbol a("a");
 	symbol x0("x0"), x1("x1"), x2("x2");
 	symbol y0("y0"), y1("y1"), y2("y2");
-	matrix A(3,3);
-	A.set(0,0,1).set(0,1,2).set(0,2,-1);
-	A.set(1,0,1).set(1,1,4).set(1,2,-2);
-	A.set(2,0,a).set(2,1,-2).set(2,2,2);
-	matrix B(3,2);
-	B.set(0,0,4).set(1,0,7).set(2,0,a);
-	B.set(0,1,0).set(1,1,0).set(2,1,4);
-	matrix X(3,2);
-	X.set(0,0,x0).set(1,0,x1).set(2,0,x2);
-	X.set(0,1,y0).set(1,1,y1).set(2,1,y2);
-	matrix cmp(3,2);
-	cmp.set(0,0,1).set(1,0,3).set(2,0,3);
-	cmp.set(0,1,0).set(1,1,2).set(2,1,4);
+	matrix A = {{1,  2, -1},
+	            {1,  4, -2},
+	            {a, -2,  2}};
+	matrix B = {{4, 0},
+	            {7, 0},
+	            {a, 4}};
+	matrix X = {{x0 ,y0},
+	            {x1, y1},
+	            {x2, y2}};
+	matrix cmp = {{1, 0},
+	              {3, 2},
+	              {3, 4}};
 	matrix sol(A.solve(X, B));
-	for (unsigned ro=0; ro<3; ++ro)
-		for (unsigned co=0; co<2; ++co)
-			if (cmp(ro,co) != sol(ro,co))
-				result = 1;
-	if (result) {
+	if (cmp != sol) {
 		clog << "Solving " << A << " * " << X << " == " << B << endl
 		     << "erroneously returned " << sol << endl;
+		result = 1;
 	}
-	
+
 	return result;
 }
 
@@ -224,16 +218,12 @@ static unsigned matrix_evalm()
 {
 	unsigned result = 0;
 
-	matrix S(2, 2, lst(
-		1, 2,
-		3, 4
-	)), T(2, 2, lst(
-		1, 1,
-		2, -1
-	)), R(2, 2, lst(
-		27, 14,
-		36, 26
-	));
+	matrix S {{1, 2},
+	          {3, 4}};
+	matrix T {{1, 1},
+	          {2, -1}};
+	matrix R {{27, 14},
+	          {36, 26}};
 
 	ex e = ((S + T) * (S + 2*T));
 	ex f = e.evalm();
@@ -258,18 +248,18 @@ static unsigned matrix_rank()
 	}
 
 	// a trivial rank one example
-	m = 1, 0, 0,
-	    2, 0, 0,
-	    3, 0, 0;
+	m = {{1, 0, 0},
+	     {2, 0, 0},
+	     {3, 0, 0}};
 	if (m.rank() != 1) {
 		clog << "The rank of " << m << " was not computed correctly." << endl;
 		++result;
 	}
 
 	// an example from Maple's help with rank two
-	m = x,  1,  0,
-	    0,  0,  1,
-	   x*y, y,  1;
+	m = {{x,   1,  0},
+	     {0,   0,  1},
+	     {x*y, y,  1}};
 	if (m.rank() != 2) {
 		clog << "The rank of " << m << " was not computed correctly." << endl;
 		++result;
@@ -288,10 +278,9 @@ static unsigned matrix_rank()
 static unsigned matrix_misc()
 {
 	unsigned result = 0;
-	matrix m1(2,2);
 	symbol a("a"), b("b"), c("c"), d("d"), e("e"), f("f");
-	m1.set(0,0,a).set(0,1,b);
-	m1.set(1,0,c).set(1,1,d);
+	matrix m1 = {{a, b},
+		     {c, d}};
 	ex tr = trace(m1);
 	
 	// check a simple trace
@@ -308,10 +297,9 @@ static unsigned matrix_misc()
 			 << " erroneously returned " << m2 << endl;
 		++result;
 	}
-	matrix m3(3,2);
-	m3.set(0,0,a).set(0,1,b);
-	m3.set(1,0,c).set(1,1,d);
-	m3.set(2,0,e).set(2,1,f);
+	matrix m3 = {{a, b},
+		     {c, d},
+		     {e, f}};
 	if (transpose(transpose(m3)) != m3) {
 		clog << "transposing 3x2 matrix " << m3 << " twice"
 		     << " erroneously returned " << transpose(transpose(m3)) << endl;
