@@ -35,16 +35,16 @@ static unsigned toeplitz_det(unsigned size)
 {
 	unsigned result = 0;
 	const symbol a("a"), b("b");
-	ex p[10] = {ex("a",lst(a,b)),
-	           ex("b",lst(a,b)),
-	           ex("a+b",lst(a,b)),
-	           ex("a^2+a*b+b^2",lst(a,b)),
-	           ex("a^3+a^2*b-a*b^2+b^3",lst(a,b)),
-	           ex("a^4+a^3*b+a^2*b^2+a*b^3+b^4",lst(a,b)),
-	           ex("a^5+a^4*b+a^3*b^2-a^2*b^3+a*b^4+b^5",lst(a,b)),
-	           ex("a^6+a^5*b+a^4*b^2+a^3*b^3+a^2*b^4+a*b^5+b^6",lst(a,b)),
-	           ex("a^7+a^6*b+a^5*b^2+a^4*b^3-a^3*b^4+a^2*b^5+a*b^6+b^7",lst(a,b)),
-	           ex("a^8+a^7*b+a^6*b^2+a^5*b^3+a^4*b^4+a^3*b^5+a^2*b^6+a*b^7+b^8",lst(a,b))
+	ex p[10] = {ex("a",lst{a,b}),
+	           ex("b",lst{a,b}),
+	           ex("a+b",lst{a,b}),
+	           ex("a^2+a*b+b^2",lst{a,b}),
+	           ex("a^3+a^2*b-a*b^2+b^3",lst{a,b}),
+	           ex("a^4+a^3*b+a^2*b^2+a*b^3+b^4",lst{a,b}),
+	           ex("a^5+a^4*b+a^3*b^2-a^2*b^3+a*b^4+b^5",lst{a,b}),
+	           ex("a^6+a^5*b+a^4*b^2+a^3*b^3+a^2*b^4+a*b^5+b^6",lst{a,b}),
+	           ex("a^7+a^6*b+a^5*b^2+a^4*b^3-a^3*b^4+a^2*b^5+a*b^6+b^7",lst{a,b}),
+	           ex("a^8+a^7*b+a^6*b^2+a^5*b^3+a^4*b^4+a^3*b^5+a^2*b^6+a*b^7+b^8",lst{a,b})
 	};
 
 	// construct Toeplitz matrix (diagonal structure: [[x,y,z],[y,x,y],[z,y,x]]):
