@@ -89,11 +89,17 @@ mul::mul(const epvector & v, const ex & oc, bool do_index_renaming)
 	GINAC_ASSERT(is_canonical());
 }
 
-mul::mul(std::auto_ptr<epvector> vp, const ex & oc, bool do_index_renaming)
+mul::mul(epvector && vp)
+{
+	overall_coeff = _ex1;
+	construct_from_epvector(std::move(vp));
+	GINAC_ASSERT(is_canonical());
+}
+
+mul::mul(epvector && vp, const ex & oc, bool do_index_renaming)
 {
-	GINAC_ASSERT(vp.get()!=0);
 	overall_coeff = oc;
-	construct_from_epvector(*vp, do_index_renaming);
+	construct_from_epvector(std::move(vp), do_index_renaming);
 	GINAC_ASSERT(is_canonical());
 }
 
@@ -146,15 +152,13 @@ void mul::do_print(const print_context & c, unsigned level) const
 
 	print_overall_coeff(c, "*");
 
-	epvector::const_iterator it = seq.begin(), itend = seq.end();
 	bool first = true;
-	while (it != itend) {
+	for (auto & it : seq) {
 		if (!first)
 			c.s << '*';
 		else
 			first = false;
-		recombine_pair_to_ex(*it).print(c, precedence());
-		++it;
+		recombine_pair_to_ex(it).print(c, precedence());
 	}
 
 	if (precedence() <= level)
@@ -170,15 +174,13 @@ void mul::do_print_latex(const print_latex & c, unsigned level) const
 
 	// Separate factors into those with negative numeric exponent
 	// and all others
-	epvector::const_iterator it = seq.begin(), itend = seq.end();
 	exvector neg_powers, others;
-	while (it != itend) {
-		GINAC_ASSERT(is_exactly_a<numeric>(it->coeff));
-		if (ex_to<numeric>(it->coeff).is_negative())
-			neg_powers.push_back(recombine_pair_to_ex(expair(it->rest, -(it->coeff))));
+	for (auto & it : seq) {
+		GINAC_ASSERT(is_exactly_a<numeric>(it.coeff));
+		if (ex_to<numeric>(it.coeff).is_negative())
+			neg_powers.push_back(recombine_pair_to_ex(expair(it.rest, -it.coeff)));
 		else
-			others.push_back(recombine_pair_to_ex(*it));
-		++it;
+			others.push_back(recombine_pair_to_ex(it));
 	}
 
 	if (!neg_powers.empty()) {
@@ -193,11 +195,9 @@ void mul::do_print_latex(const print_latex & c, unsigned level) const
 	} else {
 
 		// All other factors are printed in the ordinary way
-		exvector::const_iterator vit = others.begin(), vitend = others.end();
-		while (vit != vitend) {
+		for (auto & vit : others) {
 			c.s << ' ';
-			vit->print(c, precedence());
-			++vit;
+			vit.print(c, precedence());
 		}
 	}
 
@@ -220,7 +220,7 @@ void mul::do_print_csrc(const print_csrc & c, unsigned level) const
 	}
 
 	// Print arguments, separated by "*" or "/"
-	epvector::const_iterator it = seq.begin(), itend = seq.end();
+	auto it = seq.begin(), itend = seq.end();
 	while (it != itend) {
 
 		// If the first argument is a negative integer power, it gets printed as "1.0/<expr>"
@@ -237,11 +237,9 @@ void mul::do_print_csrc(const print_csrc & c, unsigned level) const
 		if (it->coeff.is_equal(_ex1) || it->coeff.is_equal(_ex_1))
 			it->rest.print(c, precedence());
 		else if (it->coeff.info(info_flags::negint))
-			// Outer parens around ex needed for broken GCC parser:
-			(ex(power(it->rest, -ex_to<numeric>(it->coeff)))).print(c, level);
+			ex(power(it->rest, -ex_to<numeric>(it->coeff))).print(c, level);
 		else
-			// Outer parens around ex needed for broken GCC parser:
-			(ex(power(it->rest, ex_to<numeric>(it->coeff)))).print(c, level);
+			ex(power(it->rest, ex_to<numeric>(it->coeff))).print(c, level);
 
 		if (needclosingparenthesis)
 			c.s << ")";
@@ -286,25 +284,14 @@ bool mul::info(unsigned inf) const
 		case info_flags::even:
 		case info_flags::crational_polynomial:
 		case info_flags::rational_function: {
-			epvector::const_iterator i = seq.begin(), end = seq.end();
-			while (i != end) {
-				if (!(recombine_pair_to_ex(*i).info(inf)))
+			for (auto & it : seq) {
+				if (!recombine_pair_to_ex(it).info(inf))
 					return false;
-				++i;
 			}
 			if (overall_coeff.is_equal(*_num1_p) && inf == info_flags::even)
 				return true;
 			return overall_coeff.info(inf);
 		}
-		case info_flags::algebraic: {
-			epvector::const_iterator i = seq.begin(), end = seq.end();
-			while (i != end) {
-				if ((recombine_pair_to_ex(*i).info(inf)))
-					return true;
-				++i;
-			}
-			return false;
-		}
 		case info_flags::positive:
 		case info_flags::negative: {
 			if ((inf==info_flags::positive) && (flags & status_flags::is_positive))
@@ -315,9 +302,8 @@ bool mul::info(unsigned inf) const
 				return false;
 
 			bool pos = true;
-			epvector::const_iterator i = seq.begin(), end = seq.end();
-			while (i != end) {
-				const ex& factor = recombine_pair_to_ex(*i++);
+			for (auto & it : seq) {
+				const ex& factor = recombine_pair_to_ex(it);
 				if (factor.info(info_flags::positive))
 					continue;
 				else if (factor.info(info_flags::negative))
@@ -334,9 +320,8 @@ bool mul::info(unsigned inf) const
 			if  (flags & status_flags::is_positive)
 				return true;
 			bool pos = true;
-			epvector::const_iterator i = seq.begin(), end = seq.end();
-			while (i != end) {
-				const ex& factor = recombine_pair_to_ex(*i++);
+			for (auto & it : seq) {
+				const ex& factor = recombine_pair_to_ex(it);
 				if (factor.info(info_flags::nonnegative) || factor.info(info_flags::positive))
 					continue;
 				else if (factor.info(info_flags::negative))
@@ -349,9 +334,8 @@ bool mul::info(unsigned inf) const
 		case info_flags::posint:
 		case info_flags::negint: {
 			bool pos = true;
-			epvector::const_iterator i = seq.begin(), end = seq.end();
-			while (i != end) {
-				const ex& factor = recombine_pair_to_ex(*i++);
+			for (auto & it : seq) {
+				const ex& factor = recombine_pair_to_ex(it);
 				if (factor.info(info_flags::posint))
 					continue;
 				else if (factor.info(info_flags::negint))
@@ -367,9 +351,8 @@ bool mul::info(unsigned inf) const
 		}
 		case info_flags::nonnegint: {
 			bool pos = true;
-			epvector::const_iterator i = seq.begin(), end = seq.end();
-			while (i != end) {
-				const ex& factor = recombine_pair_to_ex(*i++);
+			for (auto & it : seq) {
+				const ex& factor = recombine_pair_to_ex(it);
 				if (factor.info(info_flags::nonnegint) || factor.info(info_flags::posint))
 					continue;
 				else if (factor.info(info_flags::negint))
@@ -388,12 +371,10 @@ bool mul::info(unsigned inf) const
 				return true;
 			if (flags & (status_flags::is_positive | status_flags::is_negative))
 				return false;
-			epvector::const_iterator i = seq.begin(), end = seq.end();
-			while (i != end) {
-				const ex& term = recombine_pair_to_ex(*i);
+			for (auto & it : seq) {
+				const ex& term = recombine_pair_to_ex(it);
 				if (term.info(info_flags::positive) || term.info(info_flags::negative))
 					return false;
-				++i;
 			}
 			setflag(status_flags::purely_indefinite);
 			return true;
@@ -404,9 +385,9 @@ bool mul::info(unsigned inf) const
 
 bool mul::is_polynomial(const ex & var) const
 {
-	for (epvector::const_iterator i=seq.begin(); i!=seq.end(); ++i) {
-		if (!i->rest.is_polynomial(var) ||
-		    (i->rest.has(var) && !i->coeff.info(info_flags::nonnegint))) {
+	for (auto & it : seq) {
+		if (!it.rest.is_polynomial(var) ||
+		    (it.rest.has(var) && !it.coeff.info(info_flags::nonnegint))) {
 			return false;
 		}
 	}
@@ -417,15 +398,13 @@ int mul::degree(const ex & s) const
 {
 	// Sum up degrees of factors
 	int deg_sum = 0;
-	epvector::const_iterator i = seq.begin(), end = seq.end();
-	while (i != end) {
-		if (ex_to<numeric>(i->coeff).is_integer())
-			deg_sum += recombine_pair_to_ex(*i).degree(s);
+	for (auto & it : seq) {
+		if (ex_to<numeric>(it.coeff).is_integer())
+			deg_sum += recombine_pair_to_ex(it).degree(s);
 		else {
-			if (i->rest.has(s))
+			if (it.rest.has(s))
 				throw std::runtime_error("mul::degree() undefined degree because of non-integer exponent");
 		}
-		++i;
 	}
 	return deg_sum;
 }
@@ -434,15 +413,13 @@ int mul::ldegree(const ex & s) const
 {
 	// Sum up degrees of factors
 	int deg_sum = 0;
-	epvector::const_iterator i = seq.begin(), end = seq.end();
-	while (i != end) {
-		if (ex_to<numeric>(i->coeff).is_integer())
-			deg_sum += recombine_pair_to_ex(*i).ldegree(s);
+	for (auto & it : seq) {
+		if (ex_to<numeric>(it.coeff).is_integer())
+			deg_sum += recombine_pair_to_ex(it).ldegree(s);
 		else {
-			if (i->rest.has(s))
+			if (it.rest.has(s))
 				throw std::runtime_error("mul::ldegree() undefined degree because of non-integer exponent");
 		}
-		++i;
 	}
 	return deg_sum;
 }
@@ -455,19 +432,15 @@ ex mul::coeff(const ex & s, int n) const
 	if (n==0) {
 		// product of individual coeffs
 		// if a non-zero power of s is found, the resulting product will be 0
-		epvector::const_iterator i = seq.begin(), end = seq.end();
-		while (i != end) {
-			coeffseq.push_back(recombine_pair_to_ex(*i).coeff(s,n));
-			++i;
-		}
+		for (auto & it : seq)
+			coeffseq.push_back(recombine_pair_to_ex(it).coeff(s,n));
 		coeffseq.push_back(overall_coeff);
-		return (new mul(coeffseq))->setflag(status_flags::dynallocated);
+		return dynallocate<mul>(coeffseq);
 	}
 	
-	epvector::const_iterator i = seq.begin(), end = seq.end();
 	bool coeff_found = false;
-	while (i != end) {
-		ex t = recombine_pair_to_ex(*i);
+	for (auto & it : seq) {
+		ex t = recombine_pair_to_ex(it);
 		ex c = t.coeff(s, n);
 		if (!c.is_zero()) {
 			coeffseq.push_back(c);
@@ -475,11 +448,10 @@ ex mul::coeff(const ex & s, int n) const
 		} else {
 			coeffseq.push_back(t);
 		}
-		++i;
 	}
 	if (coeff_found) {
 		coeffseq.push_back(overall_coeff);
-		return (new mul(coeffseq))->setflag(status_flags::dynallocated);
+		return dynallocate<mul>(coeffseq);
 	}
 	
 	return _ex0;
@@ -492,23 +464,21 @@ ex mul::coeff(const ex & s, int n) const
  *  - *(+(x1,x2,...);c) -> *(+(*(x1,c),*(x2,c),...))
  *  - *(x;1) -> x
  *  - *(;c) -> c
- *
- *  @param level cut-off in recursive evaluation */
-ex mul::eval(int level) const
+ */
+ex mul::eval() const
 {
-	std::auto_ptr<epvector> evaled_seqp = evalchildren(level);
-	if (unlikely(evaled_seqp.get() != 0)) {
-		// do more evaluation later
-		return (new mul(evaled_seqp, overall_coeff))->
-		           setflag(status_flags::dynallocated);
-	}
-	
 	if (flags & status_flags::evaluated) {
 		GINAC_ASSERT(seq.size()>0);
 		GINAC_ASSERT(seq.size()>1 || !overall_coeff.is_equal(_ex1));
 		return *this;
 	}
-	
+
+	const epvector evaled = evalchildren();
+	if (unlikely(!evaled.empty())) {
+		// start over evaluating a new object
+		return dynallocate<mul>(std::move(evaled), overall_coeff);
+	}
+
 	size_t seq_size = seq.size();
 	if (overall_coeff.is_zero()) {
 		// *(...,x;0) -> 0
@@ -524,25 +494,21 @@ ex mul::eval(int level) const
 	           ex_to<numeric>((*seq.begin()).coeff).is_equal(*_num1_p)) {
 		// *(+(x,y,...);c) -> +(*(x,c),*(y,c),...) (c numeric(), no powers of +())
 		const add & addref = ex_to<add>((*seq.begin()).rest);
-		std::auto_ptr<epvector> distrseq(new epvector);
-		distrseq->reserve(addref.seq.size());
-		epvector::const_iterator i = addref.seq.begin(), end = addref.seq.end();
-		while (i != end) {
-			distrseq->push_back(addref.combine_pair_with_coeff_to_pair(*i, overall_coeff));
-			++i;
+		epvector distrseq;
+		distrseq.reserve(addref.seq.size());
+		for (auto & it : addref.seq) {
+			distrseq.push_back(addref.combine_pair_with_coeff_to_pair(it, overall_coeff));
 		}
-		return (new add(distrseq,
-		                ex_to<numeric>(addref.overall_coeff).
-		                mul_dyn(ex_to<numeric>(overall_coeff)))
-		       )->setflag(status_flags::dynallocated | status_flags::evaluated);
+		return dynallocate<add>(std::move(distrseq),
+		                        ex_to<numeric>(addref.overall_coeff).mul_dyn(ex_to<numeric>(overall_coeff)))
+			.setflag(status_flags::evaluated);
 	} else if ((seq_size >= 2) && (! (flags & status_flags::expanded))) {
 		// Strip the content and the unit part from each term. Thus
 		// things like (-x+a)*(3*x-3*a) automagically turn into - 3*(x-a)^2
 
-		epvector::const_iterator last = seq.end();
-		epvector::const_iterator i = seq.begin();
-		epvector::const_iterator j = seq.begin();
-		std::auto_ptr<epvector> s(new epvector);
+		auto i = seq.begin(), last = seq.end();
+		auto j = seq.begin();
+		epvector s;
 		numeric oc = *_num1_p;
 		bool something_changed = false;
 		while (i!=last) {
@@ -569,12 +535,12 @@ ex mul::eval(int level) const
 			}
 
 			if (! something_changed) {
-				s->reserve(seq_size);
+				s.reserve(seq_size);
 				something_changed = true;
 			}
 
 			while ((j!=i) && (j!=last)) {
-				s->push_back(*j);
+				s.push_back(*j);
 				++j;
 			}
 
@@ -585,61 +551,48 @@ ex mul::eval(int level) const
 
 			// divide add by the number in place to save at least 2 .eval() calls
 			const add& addref = ex_to<add>(i->rest);
-			add* primitive = new add(addref);
-			primitive->setflag(status_flags::dynallocated);
-			primitive->clearflag(status_flags::hash_calculated);
-			primitive->overall_coeff = ex_to<numeric>(primitive->overall_coeff).div_dyn(c);
-			for (epvector::iterator ai = primitive->seq.begin(); ai != primitive->seq.end(); ++ai)
-				ai->coeff = ex_to<numeric>(ai->coeff).div_dyn(c);
-			
-			s->push_back(expair(*primitive, _ex1));
+			add & primitive = dynallocate<add>(addref);
+			primitive.clearflag(status_flags::hash_calculated);
+			primitive.overall_coeff = ex_to<numeric>(primitive.overall_coeff).div_dyn(c);
+			for (auto & ai : primitive.seq)
+				ai.coeff = ex_to<numeric>(ai.coeff).div_dyn(c);
+
+			s.push_back(expair(primitive, _ex1));
 
 			++i;
 			++j;
 		}
 		if (something_changed) {
 			while (j!=last) {
-				s->push_back(*j);
+				s.push_back(*j);
 				++j;
 			}
-			return (new mul(s, ex_to<numeric>(overall_coeff).mul_dyn(oc))
-			       )->setflag(status_flags::dynallocated);
+			return dynallocate<mul>(std::move(s), ex_to<numeric>(overall_coeff).mul_dyn(oc));
 		}
 	}
 
 	return this->hold();
 }
 
-ex mul::evalf(int level) const
+ex mul::evalf() const
 {
-	if (level==1)
-		return mul(seq,overall_coeff);
-	
-	if (level==-max_recursion_level)
-		throw(std::runtime_error("max recursion level reached"));
-	
-	std::auto_ptr<epvector> s(new epvector);
-	s->reserve(seq.size());
+	epvector s;
+	s.reserve(seq.size());
 
-	--level;
-	epvector::const_iterator i = seq.begin(), end = seq.end();
-	while (i != end) {
-		s->push_back(combine_ex_with_coeff_to_pair(i->rest.evalf(level),
-		                                           i->coeff));
-		++i;
-	}
-	return mul(s, overall_coeff.evalf(level));
+	for (auto & it : seq)
+		s.push_back(expair(it.rest.evalf(), it.coeff));
+	return dynallocate<mul>(std::move(s), overall_coeff.evalf());
 }
 
 void mul::find_real_imag(ex & rp, ex & ip) const
 {
 	rp = overall_coeff.real_part();
 	ip = overall_coeff.imag_part();
-	for (epvector::const_iterator i=seq.begin(); i!=seq.end(); ++i) {
-		ex factor = recombine_pair_to_ex(*i);
+	for (auto & it : seq) {
+		ex factor = recombine_pair_to_ex(it);
 		ex new_rp = factor.real_part();
 		ex new_ip = factor.imag_part();
-		if(new_ip.is_zero()) {
+		if (new_ip.is_zero()) {
 			rp *= new_rp;
 			ip *= new_rp;
 		} else {
@@ -676,21 +629,19 @@ ex mul::evalm() const
 	// Evaluate children first, look whether there are any matrices at all
 	// (there can be either no matrices or one matrix; if there were more
 	// than one matrix, it would be a non-commutative product)
-	std::auto_ptr<epvector> s(new epvector);
-	s->reserve(seq.size());
+	epvector s;
+	s.reserve(seq.size());
 
 	bool have_matrix = false;
 	epvector::iterator the_matrix;
 
-	epvector::const_iterator i = seq.begin(), end = seq.end();
-	while (i != end) {
-		const ex &m = recombine_pair_to_ex(*i).evalm();
-		s->push_back(split_ex_to_pair(m));
+	for (auto & it : seq) {
+		const ex &m = recombine_pair_to_ex(it).evalm();
+		s.push_back(split_ex_to_pair(m));
 		if (is_a<matrix>(m)) {
 			have_matrix = true;
-			the_matrix = s->end() - 1;
+			the_matrix = s.end() - 1;
 		}
-		++i;
 	}
 
 	if (have_matrix) {
@@ -698,12 +649,12 @@ ex mul::evalm() const
 		// The product contained a matrix. We will multiply all other factors
 		// into that matrix.
 		matrix m = ex_to<matrix>(the_matrix->rest);
-		s->erase(the_matrix);
-		ex scalar = (new mul(s, overall_coeff))->setflag(status_flags::dynallocated);
+		s.erase(the_matrix);
+		ex scalar = dynallocate<mul>(std::move(s), overall_coeff);
 		return m.mul_scalar(scalar);
 
 	} else
-		return (new mul(s, overall_coeff))->setflag(status_flags::dynallocated);
+		return dynallocate<mul>(std::move(s), overall_coeff);
 }
 
 ex mul::eval_ncmul(const exvector & v) const
@@ -712,12 +663,9 @@ ex mul::eval_ncmul(const exvector & v) const
 		return inherited::eval_ncmul(v);
 
 	// Find first noncommutative element and call its eval_ncmul()
-	epvector::const_iterator i = seq.begin(), end = seq.end();
-	while (i != end) {
-		if (i->rest.return_type() == return_types::noncommutative)
-			return i->rest.eval_ncmul(v);
-		++i;
-	}
+	for (auto & it : seq)
+		if (it.rest.return_type() == return_types::noncommutative)
+			return it.rest.eval_ncmul(v);
 	return inherited::eval_ncmul(v);
 }
 
@@ -825,26 +773,26 @@ ex mul::algebraic_subs_mul(const exmap & m, unsigned options) const
 	ex divide_by = 1;
 	ex multiply_by = 1;
 
-	for (exmap::const_iterator it = m.begin(); it != m.end(); ++it) {
+	for (auto & it : m) {
 
-		if (is_exactly_a<mul>(it->first)) {
+		if (is_exactly_a<mul>(it.first)) {
 retry1:
 			int nummatches = std::numeric_limits<int>::max();
 			std::vector<bool> currsubsed(nops(), false);
 			exmap repls;
 			
-			if(!algebraic_match_mul_with_mul(*this, it->first, repls, 0, nummatches, subsed, currsubsed))
+			if (!algebraic_match_mul_with_mul(*this, it.first, repls, 0, nummatches, subsed, currsubsed))
 				continue;
 
 			for (size_t j=0; j<subsed.size(); j++)
 				if (currsubsed[j])
 					subsed[j] = true;
 			ex subsed_pattern
-				= it->first.subs(repls, subs_options::no_pattern);
-			divide_by *= power(subsed_pattern, nummatches);
+				= it.first.subs(repls, subs_options::no_pattern);
+			divide_by *= pow(subsed_pattern, nummatches);
 			ex subsed_result
-				= it->second.subs(repls, subs_options::no_pattern);
-			multiply_by *= power(subsed_result, nummatches);
+				= it.second.subs(repls, subs_options::no_pattern);
+			multiply_by *= pow(subsed_result, nummatches);
 			goto retry1;
 
 		} else {
@@ -852,14 +800,14 @@ retry1:
 			for (size_t j=0; j<this->nops(); j++) {
 				int nummatches = std::numeric_limits<int>::max();
 				exmap repls;
-				if (!subsed[j] && tryfactsubs(op(j), it->first, nummatches, repls)){
+				if (!subsed[j] && tryfactsubs(op(j), it.first, nummatches, repls)){
 					subsed[j] = true;
 					ex subsed_pattern
-						= it->first.subs(repls, subs_options::no_pattern);
-					divide_by *= power(subsed_pattern, nummatches);
+						= it.first.subs(repls, subs_options::no_pattern);
+					divide_by *= pow(subsed_pattern, nummatches);
 					ex subsed_result
-						= it->second.subs(repls, subs_options::no_pattern);
-					multiply_by *= power(subsed_result, nummatches);
+						= it.second.subs(repls, subs_options::no_pattern);
+					multiply_by *= pow(subsed_result, nummatches);
 				}
 			}
 		}
@@ -882,8 +830,8 @@ ex mul::conjugate() const
 {
 	// The base class' method is wrong here because we have to be careful at
 	// branch cuts. power::conjugate takes care of that already, so use it.
-	epvector *newepv = 0;
-	for (epvector::const_iterator i=seq.begin(); i!=seq.end(); ++i) {
+	std::unique_ptr<epvector> newepv(nullptr);
+	for (auto i=seq.begin(); i!=seq.end(); ++i) {
 		if (newepv) {
 			newepv->push_back(split_ex_to_pair(recombine_pair_to_ex(*i).conjugate()));
 			continue;
@@ -893,9 +841,9 @@ ex mul::conjugate() const
 		if (c.is_equal(x)) {
 			continue;
 		}
-		newepv = new epvector;
+		newepv.reset(new epvector);
 		newepv->reserve(seq.size());
-		for (epvector::const_iterator j=seq.begin(); j!=i; ++j) {
+		for (auto j=seq.begin(); j!=i; ++j) {
 			newepv->push_back(*j);
 		}
 		newepv->push_back(split_ex_to_pair(c));
@@ -904,9 +852,7 @@ ex mul::conjugate() const
 	if (!newepv && are_ex_trivially_equal(x, overall_coeff)) {
 		return *this;
 	}
-	ex result = thisexpairseq(newepv ? *newepv : seq, x);
-	delete newepv;
-	return result;
+	return thisexpairseq(newepv ? std::move(*newepv) : seq, x);
 }
 
 
@@ -922,17 +868,17 @@ ex mul::derivative(const symbol & s) const
 	
 	// D(a*b*c) = D(a)*b*c + a*D(b)*c + a*b*D(c)
 	epvector mulseq = seq;
-	epvector::const_iterator i = seq.begin(), end = seq.end();
-	epvector::iterator i2 = mulseq.begin();
+	auto i = seq.begin(), end = seq.end();
+	auto i2 = mulseq.begin();
 	while (i != end) {
-		expair ep = split_ex_to_pair(power(i->rest, i->coeff - _ex1) *
+		expair ep = split_ex_to_pair(pow(i->rest, i->coeff - _ex1) *
 		                             i->rest.diff(s));
 		ep.swap(*i2);
-		addseq.push_back((new mul(mulseq, overall_coeff * i->coeff))->setflag(status_flags::dynallocated));
+		addseq.push_back(dynallocate<mul>(mulseq, overall_coeff * i->coeff));
 		ep.swap(*i2);
 		++i; ++i2;
 	}
-	return (new add(addseq))->setflag(status_flags::dynallocated);
+	return dynallocate<add>(addseq);
 }
 
 int mul::compare_same_type(const basic & other) const
@@ -979,24 +925,22 @@ return_type_t mul::return_type_tinfo() const
 		return make_return_type_t<mul>(); // mul without factors: should not happen
 	
 	// return type_info of first noncommutative element
-	epvector::const_iterator i = seq.begin(), end = seq.end();
-	while (i != end) {
-		if (i->rest.return_type() == return_types::noncommutative)
-			return i->rest.return_type_tinfo();
-		++i;
-	}
+	for (auto & it : seq)
+		if (it.rest.return_type() == return_types::noncommutative)
+			return it.rest.return_type_tinfo();
+
 	// no noncommutative element found, should not happen
 	return make_return_type_t<mul>();
 }
 
 ex mul::thisexpairseq(const epvector & v, const ex & oc, bool do_index_renaming) const
 {
-	return (new mul(v, oc, do_index_renaming))->setflag(status_flags::dynallocated);
+	return dynallocate<mul>(v, oc, do_index_renaming);
 }
 
-ex mul::thisexpairseq(std::auto_ptr<epvector> vp, const ex & oc, bool do_index_renaming) const
+ex mul::thisexpairseq(epvector && vp, const ex & oc, bool do_index_renaming) const
 {
-	return (new mul(vp, oc, do_index_renaming))->setflag(status_flags::dynallocated);
+	return dynallocate<mul>(std::move(vp), oc, do_index_renaming);
 }
 
 expair mul::split_ex_to_pair(const ex & e) const
@@ -1012,35 +956,52 @@ expair mul::split_ex_to_pair(const ex & e) const
 expair mul::combine_ex_with_coeff_to_pair(const ex & e,
                                           const ex & c) const
 {
+	GINAC_ASSERT(is_exactly_a<numeric>(c));
+
+	// First, try a common shortcut:
+	if (is_exactly_a<symbol>(e))
+		return expair(e, c);
+
+	// trivial case: exponent 1
+	if (c.is_equal(_ex1))
+		return split_ex_to_pair(e);
+
 	// to avoid duplication of power simplification rules,
 	// we create a temporary power object
 	// otherwise it would be hard to correctly evaluate
 	// expression like (4^(1/3))^(3/2)
-	if (c.is_equal(_ex1))
-		return split_ex_to_pair(e);
-
-	return split_ex_to_pair(power(e,c));
+	return split_ex_to_pair(pow(e,c));
 }
 
 expair mul::combine_pair_with_coeff_to_pair(const expair & p,
                                             const ex & c) const
 {
+	GINAC_ASSERT(is_exactly_a<numeric>(p.coeff));
+	GINAC_ASSERT(is_exactly_a<numeric>(c));
+
+	// First, try a common shortcut:
+	if (is_exactly_a<symbol>(p.rest))
+		return expair(p.rest, p.coeff * c);
+
+	// trivial case: exponent 1
+	if (c.is_equal(_ex1))
+		return p;
+	if (p.coeff.is_equal(_ex1))
+		return expair(p.rest, c);
+
 	// to avoid duplication of power simplification rules,
 	// we create a temporary power object
 	// otherwise it would be hard to correctly evaluate
 	// expression like (4^(1/3))^(3/2)
-	if (c.is_equal(_ex1))
-		return p;
-
-	return split_ex_to_pair(power(recombine_pair_to_ex(p),c));
+	return split_ex_to_pair(pow(recombine_pair_to_ex(p),c));
 }
 
 ex mul::recombine_pair_to_ex(const expair & p) const
 {
-	if (ex_to<numeric>(p.coeff).is_equal(*_num1_p)) 
+	if (p.coeff.is_equal(_ex1))
 		return p.rest;
 	else
-		return (new power(p.rest,p.coeff))->setflag(status_flags::dynallocated);
+		return dynallocate<power>(p.rest, p.coeff);
 }
 
 bool mul::expair_needs_further_processing(epp it)
@@ -1097,8 +1058,8 @@ bool mul::can_make_flat(const expair & p) const
 bool mul::can_be_further_expanded(const ex & e)
 {
 	if (is_exactly_a<mul>(e)) {
-		for (epvector::const_iterator cit = ex_to<mul>(e).seq.begin(); cit != ex_to<mul>(e).seq.end(); ++cit) {
-			if (is_exactly_a<add>(cit->rest) && cit->coeff.info(info_flags::posint))
+		for (auto & it : ex_to<mul>(e).seq) {
+			if (is_exactly_a<add>(it.rest) && it.coeff.info(info_flags::posint))
 				return true;
 		}
 	} else if (is_exactly_a<power>(e)) {
@@ -1110,27 +1071,29 @@ bool mul::can_be_further_expanded(const ex & e)
 
 ex mul::expand(unsigned options) const
 {
-	{
-	// trivial case: expanding the monomial (~ 30% of all calls)
-		epvector::const_iterator i = seq.begin(), seq_end = seq.end();
-		while ((i != seq.end()) &&  is_a<symbol>(i->rest) && i->coeff.info(info_flags::integer))
-			++i;
-		if (i == seq_end) {
-			setflag(status_flags::expanded);
-			return *this;
+	// Check for trivial case: expanding the monomial (~ 30% of all calls)
+	bool monomial_case = true;
+	for (const auto & i : seq) {
+		if (!is_a<symbol>(i.rest) || !i.coeff.info(info_flags::integer)) {
+			monomial_case = false;
+			break;
 		}
 	}
+	if (monomial_case) {
+		setflag(status_flags::expanded);
+		return *this;
+	}
 
 	// do not rename indices if the object has no indices at all
 	if ((!(options & expand_options::expand_rename_idx)) && 
-			this->info(info_flags::has_indices))
+	    this->info(info_flags::has_indices))
 		options |= expand_options::expand_rename_idx;
 
 	const bool skip_idx_rename = !(options & expand_options::expand_rename_idx);
 
 	// First, expand the children
-	std::auto_ptr<epvector> expanded_seqp = expandchildren(options);
-	const epvector & expanded_seq = (expanded_seqp.get() ? *expanded_seqp : seq);
+	epvector expanded = expandchildren(options);
+	const epvector & expanded_seq = (expanded.empty() ? seq : expanded);
 
 	// Now, look for all the factors that are sums and multiply each one out
 	// with the next one that is found while collecting the factors which are
@@ -1140,58 +1103,54 @@ ex mul::expand(unsigned options) const
 	epvector non_adds;
 	non_adds.reserve(expanded_seq.size());
 
-	for (epvector::const_iterator cit = expanded_seq.begin(); cit != expanded_seq.end(); ++cit) {
-		if (is_exactly_a<add>(cit->rest) &&
-			(cit->coeff.is_equal(_ex1))) {
+	for (const auto & cit : expanded_seq) {
+		if (is_exactly_a<add>(cit.rest) &&
+			(cit.coeff.is_equal(_ex1))) {
 			if (is_exactly_a<add>(last_expanded)) {
 
 				// Expand a product of two sums, aggressive version.
 				// Caring for the overall coefficients in separate loops can
 				// sometimes give a performance gain of up to 15%!
 
-				const int sizedifference = ex_to<add>(last_expanded).seq.size()-ex_to<add>(cit->rest).seq.size();
+				const int sizedifference = ex_to<add>(last_expanded).seq.size()-ex_to<add>(cit.rest).seq.size();
 				// add2 is for the inner loop and should be the bigger of the two sums
 				// in the presence of asymptotically good sorting:
-				const add& add1 = (sizedifference<0 ? ex_to<add>(last_expanded) : ex_to<add>(cit->rest));
-				const add& add2 = (sizedifference<0 ? ex_to<add>(cit->rest) : ex_to<add>(last_expanded));
-				const epvector::const_iterator add1begin = add1.seq.begin();
-				const epvector::const_iterator add1end   = add1.seq.end();
-				const epvector::const_iterator add2begin = add2.seq.begin();
-				const epvector::const_iterator add2end   = add2.seq.end();
+				const add& add1 = (sizedifference<0 ? ex_to<add>(last_expanded) : ex_to<add>(cit.rest));
+				const add& add2 = (sizedifference<0 ? ex_to<add>(cit.rest) : ex_to<add>(last_expanded));
 				epvector distrseq;
 				distrseq.reserve(add1.seq.size()+add2.seq.size());
 
 				// Multiply add2 with the overall coefficient of add1 and append it to distrseq:
 				if (!add1.overall_coeff.is_zero()) {
 					if (add1.overall_coeff.is_equal(_ex1))
-						distrseq.insert(distrseq.end(),add2begin,add2end);
+						distrseq.insert(distrseq.end(), add2.seq.begin(), add2.seq.end());
 					else
-						for (epvector::const_iterator i=add2begin; i!=add2end; ++i)
-							distrseq.push_back(expair(i->rest, ex_to<numeric>(i->coeff).mul_dyn(ex_to<numeric>(add1.overall_coeff))));
+						for (const auto & i : add2.seq)
+							distrseq.push_back(expair(i.rest, ex_to<numeric>(i.coeff).mul_dyn(ex_to<numeric>(add1.overall_coeff))));
 				}
 
 				// Multiply add1 with the overall coefficient of add2 and append it to distrseq:
 				if (!add2.overall_coeff.is_zero()) {
 					if (add2.overall_coeff.is_equal(_ex1))
-						distrseq.insert(distrseq.end(),add1begin,add1end);
+						distrseq.insert(distrseq.end(), add1.seq.begin(), add1.seq.end());
 					else
-						for (epvector::const_iterator i=add1begin; i!=add1end; ++i)
-							distrseq.push_back(expair(i->rest, ex_to<numeric>(i->coeff).mul_dyn(ex_to<numeric>(add2.overall_coeff))));
+						for (const auto & i : add1.seq)
+							distrseq.push_back(expair(i.rest, ex_to<numeric>(i.coeff).mul_dyn(ex_to<numeric>(add2.overall_coeff))));
 				}
 
 				// Compute the new overall coefficient and put it together:
-				ex tmp_accu = (new add(distrseq, add1.overall_coeff*add2.overall_coeff))->setflag(status_flags::dynallocated);
+				ex tmp_accu = dynallocate<add>(distrseq, add1.overall_coeff*add2.overall_coeff);
 
 				exvector add1_dummy_indices, add2_dummy_indices, add_indices;
 				lst dummy_subs;
 
 				if (!skip_idx_rename) {
-					for (epvector::const_iterator i=add1begin; i!=add1end; ++i) {
-						add_indices = get_all_dummy_indices_safely(i->rest);
+					for (const auto & i : add1.seq) {
+						add_indices = get_all_dummy_indices_safely(i.rest);
 						add1_dummy_indices.insert(add1_dummy_indices.end(), add_indices.begin(), add_indices.end());
 					}
-					for (epvector::const_iterator i=add2begin; i!=add2end; ++i) {
-						add_indices = get_all_dummy_indices_safely(i->rest);
+					for (const auto & i : add2.seq) {
+						add_indices = get_all_dummy_indices_safely(i.rest);
 						add2_dummy_indices.insert(add2_dummy_indices.end(), add_indices.begin(), add_indices.end());
 					}
 
@@ -1201,37 +1160,37 @@ ex mul::expand(unsigned options) const
 				}
 
 				// Multiply explicitly all non-numeric terms of add1 and add2:
-				for (epvector::const_iterator i2=add2begin; i2!=add2end; ++i2) {
+				for (const auto & i2 : add2.seq) {
 					// We really have to combine terms here in order to compactify
 					// the result.  Otherwise it would become waayy tooo bigg.
 					numeric oc(*_num0_p);
 					epvector distrseq2;
 					distrseq2.reserve(add1.seq.size());
 					const ex i2_new = (skip_idx_rename || (dummy_subs.op(0).nops() == 0) ?
-							i2->rest :
-							i2->rest.subs(ex_to<lst>(dummy_subs.op(0)), 
-								ex_to<lst>(dummy_subs.op(1)), subs_options::no_pattern));
-					for (epvector::const_iterator i1=add1begin; i1!=add1end; ++i1) {
+							i2.rest :
+							i2.rest.subs(ex_to<lst>(dummy_subs.op(0)),
+							             ex_to<lst>(dummy_subs.op(1)), subs_options::no_pattern));
+					for (const auto & i1 : add1.seq) {
 						// Don't push_back expairs which might have a rest that evaluates to a numeric,
 						// since that would violate an invariant of expairseq:
-						const ex rest = (new mul(i1->rest, i2_new))->setflag(status_flags::dynallocated);
+						const ex rest = dynallocate<mul>(i1.rest, i2_new);
 						if (is_exactly_a<numeric>(rest)) {
-							oc += ex_to<numeric>(rest).mul(ex_to<numeric>(i1->coeff).mul(ex_to<numeric>(i2->coeff)));
+							oc += ex_to<numeric>(rest).mul(ex_to<numeric>(i1.coeff).mul(ex_to<numeric>(i2.coeff)));
 						} else {
-							distrseq2.push_back(expair(rest, ex_to<numeric>(i1->coeff).mul_dyn(ex_to<numeric>(i2->coeff))));
+							distrseq2.push_back(expair(rest, ex_to<numeric>(i1.coeff).mul_dyn(ex_to<numeric>(i2.coeff))));
 						}
 					}
-					tmp_accu += (new add(distrseq2, oc))->setflag(status_flags::dynallocated);
-				} 
+					tmp_accu += dynallocate<add>(std::move(distrseq2), oc);
+				}
 				last_expanded = tmp_accu;
 			} else {
 				if (!last_expanded.is_equal(_ex1))
 					non_adds.push_back(split_ex_to_pair(last_expanded));
-				last_expanded = cit->rest;
+				last_expanded = cit.rest;
 			}
 
 		} else {
-			non_adds.push_back(*cit);
+			non_adds.push_back(cit);
 		}
 	}
 
@@ -1253,7 +1212,7 @@ ex mul::expand(unsigned options) const
 				factors.push_back(split_ex_to_pair(last_expanded.op(i)));
 			else
 				factors.push_back(split_ex_to_pair(rename_dummy_indices_uniquely(va, last_expanded.op(i))));
-			ex term = (new mul(factors, overall_coeff))->setflag(status_flags::dynallocated);
+			ex term = dynallocate<mul>(factors, overall_coeff);
 			if (can_be_further_expanded(term)) {
 				distrseq.push_back(term.expand());
 			} else {
@@ -1263,12 +1222,11 @@ ex mul::expand(unsigned options) const
 			}
 		}
 
-		return ((new add(distrseq))->
-		        setflag(status_flags::dynallocated | (options == 0 ? status_flags::expanded : 0)));
+		return dynallocate<add>(distrseq).setflag(options == 0 ? status_flags::expanded : 0);
 	}
 
 	non_adds.push_back(split_ex_to_pair(last_expanded));
-	ex result = (new mul(non_adds, overall_coeff))->setflag(status_flags::dynallocated);
+	ex result = dynallocate<mul>(non_adds, overall_coeff);
 	if (can_be_further_expanded(result)) {
 		return result.expand();
 	} else {
@@ -1295,43 +1253,42 @@ ex mul::expand(unsigned options) const
  *  to allow for early cancellations and thus safe memory.
  *
  *  @see mul::expand()
- *  @return pointer to epvector containing expanded representation or zero
- *  pointer, if sequence is unchanged. */
-std::auto_ptr<epvector> mul::expandchildren(unsigned options) const
+ *  @return epvector containing expanded pairs, empty if no members
+ *    had to be changed. */
+epvector mul::expandchildren(unsigned options) const
 {
-	const epvector::const_iterator last = seq.end();
-	epvector::const_iterator cit = seq.begin();
+	auto cit = seq.begin(), last = seq.end();
 	while (cit!=last) {
 		const ex & factor = recombine_pair_to_ex(*cit);
 		const ex & expanded_factor = factor.expand(options);
 		if (!are_ex_trivially_equal(factor,expanded_factor)) {
 			
 			// something changed, copy seq, eval and return it
-			std::auto_ptr<epvector> s(new epvector);
-			s->reserve(seq.size());
+			epvector s;
+			s.reserve(seq.size());
 			
 			// copy parts of seq which are known not to have changed
-			epvector::const_iterator cit2 = seq.begin();
+			auto cit2 = seq.begin();
 			while (cit2!=cit) {
-				s->push_back(*cit2);
+				s.push_back(*cit2);
 				++cit2;
 			}
 
 			// copy first changed element
-			s->push_back(split_ex_to_pair(expanded_factor));
+			s.push_back(split_ex_to_pair(expanded_factor));
 			++cit2;
 
 			// copy rest
 			while (cit2!=last) {
-				s->push_back(split_ex_to_pair(recombine_pair_to_ex(*cit2).expand(options)));
+				s.push_back(split_ex_to_pair(recombine_pair_to_ex(*cit2).expand(options)));
 				++cit2;
 			}
 			return s;
 		}
 		++cit;
 	}
-	
-	return std::auto_ptr<epvector>(0); // nothing has changed
+
+	return epvector(); // nothing has changed
 }
 
 GINAC_BIND_UNARCHIVER(mul);
